package com.ruoyi.common.utils.poi;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.utils.DictUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;
import java.util.Map;

import static com.ruoyi.common.core.text.CharsetKit.UTF_8;

public class ExcelMultipleSheetsUtil {
    /**
     * 导出excel：可多个sheet页
     *
     * @param data          数据：Map 集合【key == 每一个sheet页的名称，value == sheet页数据】
     * @param excelFileName excel文件名
     * @param suffixName    后缀名
     * @param response      响应
     * @throws IOException 异常
     */
    public static void excelMultipleSheets(Map<String, Object> data, String excelFileName, String suffixName, HttpServletResponse response,Map<String,String> cover) throws IOException {
        // 创建工作簿
        try (Workbook workbook = new XSSFWorkbook()) {
            Sheet sheetFirst = workbook.createSheet("概要");
            sheetFirst.setColumnWidth(0, 25 * 256);
            sheetFirst.setColumnWidth(1, 25 * 256);
            // 创建数据行
            Row dataRowHead = sheetFirst.createRow(1);
            dataRowHead .setHeightInPoints(20);
            Cell cell = dataRowHead.createCell(0);
            dataRowHead.createCell(1).setCellStyle(getStyle(sheetFirst.getWorkbook()));
            cell.setCellValue("检测方案");
            cell.setCellStyle(getStyle(sheetFirst.getWorkbook()));
            // 合并区域：从第1行到第2行，第1列到第2列（注意：行和列的索引从0开始）
            CellRangeAddress mergeRegion = new CellRangeAddress(1, 2, 0, 1);
            // 将合并区域添加到工作表中
            sheetFirst.addMergedRegion(mergeRegion);
            final int[] i = {2};
            cover.forEach((key,value)->{
                Row dataRow  =   sheetFirst.createRow(i[0]);
                dataRow.setHeightInPoints(20);
               Cell cellTitle =  dataRow.createCell(0);
               cellTitle.setCellValue(key);
               cellTitle.setCellStyle(createCellStyle(sheetFirst.getWorkbook()));
                Cell cellText =   dataRow.createCell(1);
                cellText.setCellValue(value);
                cellText.setCellStyle(createCellStyle(sheetFirst.getWorkbook()));
                i[0] = i[0] +1;
            });

            for (Map.Entry<String, Object> entry : data.entrySet()) {
                String sheetName = entry.getKey();
                Object sheetData = entry.getValue();
                Sheet sheet = workbook.createSheet(sheetName);
                if (ObjectUtil.isNotEmpty(sheetData)) {
                    createSheetWithData(sheet, sheetData);
                }
            }

            setResponseHeader(response, excelFileName, suffixName);
            // 写出文件
            workbook.write(response.getOutputStream());
        }
    }

    private static CellStyle getStyle(Workbook workbook) {
        CellStyle style = workbook.createCellStyle();

        // 设置底边框;
        style.setBorderBottom(BorderStyle.THIN);
        // 设置底边框颜色;
        style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        // 设置左边框;
        style.setBorderLeft(BorderStyle.THIN);
        // 设置左边框颜色;
        style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        // 设置右边框;
        style.setBorderRight(BorderStyle.THIN);
        // 设置右边框颜色;
        style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        // 设置顶边框;
        style.setBorderTop(BorderStyle.THIN);
        // 设置顶边框颜色;
        style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());

        // 设置字体
        Font font = workbook.createFont();
        // 设置字体大小
        font.setFontHeightInPoints((short) 20);
        // 设置字体名字
        font.setFontName("微软雅黑");
        // 在样式用应用设置的字体;
        style.setFont(font);

        // 设置自动换行;
        style.setWrapText(true);
        // 设置水平对齐的样式为居中对齐;
        style.setAlignment(HorizontalAlignment.CENTER);
        // 设置垂直对齐的样式为居中对齐;
        style.setVerticalAlignment(VerticalAlignment.CENTER);

        // 设置填充样式（实心填充）
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        // 设置单元格颜色
        style.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        return style;
    }

    /**
     * 创建表单并填充数据
     *
     * @param sheet 表单
     * @param data  数据
     */
    private static void createSheetWithData(Sheet sheet, Object data) {
        if (data instanceof List) {
            createSheetWithListData(sheet, (List<?>) data);
        } else {
            createSheetWithObjectData(sheet, data);
        }
    }

    /**
     * 创建列表类型数据对应的Excel表单
     *
     * @param sheet    表单
     * @param dataList 数据列表
     */
    private static void createSheetWithListData(Sheet sheet, List<?> dataList) {
        if (CollUtil.isNotEmpty(dataList)) {
            Object firstItem = dataList.get(0);
            createHeaderRow(sheet, firstItem.getClass());
            int rowIndex = 1;
            for (Object item : dataList) {
                createDataRow(sheet, item, rowIndex++);
            }
        }
    }

    /**
     * 创建对象类型数据对应的Excel表单
     *
     * @param sheet 表单
     * @param data  数据
     */
    private static void createSheetWithObjectData(Sheet sheet, Object data) {
        createHeaderRow(sheet, data.getClass());
        createDataRow(sheet, data, 1);
    }

    /**
     * 创建表头行
     *
     * @param sheet 表单
     * @param clazz 数据类
     */
    private static void createHeaderRow(Sheet sheet, Class<?> clazz) {
        // 创建单元格样式
        CellStyle headerCellStyle = createCellStyle(sheet.getWorkbook());
        headerCellStyle.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
        // 填充效果（全景填充）
        headerCellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
        headerCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        // 创建标题行
        Row headerRow = sheet.createRow(0);
        Field[] fields = clazz.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            createHeaderCell(sheet, headerCellStyle, fields, headerRow, i);
        }
    }

    /**
     * 创建数据行
     *
     * @param sheet    表单
     * @param data     数据
     * @param rowIndex 行号
     */
    private static void createDataRow(Sheet sheet, Object data, int rowIndex) {
        // 创建单元格样式
        CellStyle dataCellStyle = createCellStyle(sheet.getWorkbook());

        // 创建数据行
        Row dataRow = sheet.createRow(rowIndex);
        Field[] fields = data.getClass().getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            createDataCell(dataCellStyle, fields, dataRow, i, data);
        }
    }

    /**
     * 创建单元格样式
     *
     * @param workbook 工作簿
     * @return 单元格样式
     */
    private static CellStyle createCellStyle(Workbook workbook) {
        CellStyle cellStyle = workbook.createCellStyle();

        // 设置自动换行;
        cellStyle.setWrapText(true);
        // 设置 水平和垂直 居中对齐
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

        // 设置 上 下 左 右 边框及颜色
        cellStyle.setBorderTop(BorderStyle.THIN);
        cellStyle.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        cellStyle.setBorderBottom(BorderStyle.THIN);
        cellStyle.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        cellStyle.setBorderLeft(BorderStyle.THIN);
        cellStyle.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
        cellStyle.setBorderRight(BorderStyle.THIN);
        cellStyle.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());

        // 设置字体
        Font dataFont = workbook.createFont();
        dataFont.setFontName("Arial");
        dataFont.setFontHeightInPoints((short) 10);
        cellStyle.setFont(dataFont);

        return cellStyle;
    }

    /**
     * 创建Excel表头单元格
     *
     * @param sheet           表单
     * @param headerCellStyle 单元格样式
     * @param fields          字段
     * @param headerRow       标题行
     * @param i               序号
     */
    private static void createHeaderCell(Sheet sheet, CellStyle headerCellStyle, Field[] fields, Row headerRow, int i) {
        // 默认宽度
        double width = 16;
        Excel excelAnnotation = fields[i].getAnnotation(Excel.class);
        if (excelAnnotation != null && !ObjectUtil.isEmpty(excelAnnotation.width())) {
            width = excelAnnotation.width();
        }

        // 设置宽度
        sheet.setColumnWidth(i, (int) ((width + 0.72) * 256));

        if (excelAnnotation != null) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(excelAnnotation.name());
            cell.setCellStyle(headerCellStyle);
        }
    }

    /**
     * 创建Excel数据单元格
     *
     * @param dataCellStyle 单元格样式
     * @param fields        字段
     * @param dataRow       数据行
     * @param i             序号
     * @param data          数据
     */
    private static void createDataCell(CellStyle dataCellStyle, Field[] fields, Row dataRow, int i, Object data) {
        Cell cell = dataRow.createCell(i);
        cell.setCellStyle(dataCellStyle);

        try {
            fields[i].setAccessible(true);
            Object value = fields[i].get(data);
            handleAnnotationAndSetValue(cell, fields[i], value);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    /**
     * 处理注解并设置单元格值
     *
     * @param cell  单元格
     * @param field 字段
     * @param value 值
     */
    private static void handleAnnotationAndSetValue(Cell cell, Field field, Object value) {
        if (field.isAnnotationPresent(Excel.class) && field.getAnnotation(Excel.class).dictType().length() > 0) {
            value = DictUtils.getDictLabel(field.getAnnotation(Excel.class).dictType(), String.valueOf(value));
        }
        if (field.isAnnotationPresent(Excel.class) && StrUtil.isNotEmpty(field.getAnnotation(Excel.class).dateFormat())) {
            value = DateUtil.format(Convert.convert(Date.class, value), field.getAnnotation(Excel.class).dateFormat());
        }
        cell.setCellValue(ObjectUtil.isEmpty(value) ? null : value.toString());
    }

    /**
     * 设置响应头
     *
     * @param response      响应
     * @param excelFileName 文件名
     * @param suffixName    后缀名
     * @throws UnsupportedEncodingException 编码异常
     */
    private static void setResponseHeader(HttpServletResponse response, String excelFileName, String suffixName) throws UnsupportedEncodingException {
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(excelFileName + suffixName, UTF_8));
    }

}
